@@ -1,4 +1,4 @@
-/* $Id: control.cc 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: control.cc 738 2010-09-02 07:21:40Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -154,6 +154,13 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    NetPerfMeterAddFlowMessage* addFlowMsg = (NetPerfMeterAddFlowMessage*)&addFlowMsgBuffer;
    addFlowMsg->Header.Type   = NETPERFMETER_ADD_FLOW;
    addFlowMsg->Header.Flags  = 0x00;
+   if(flow->getTrafficSpec().UseCMT) {
+      addFlowMsg->Header.Flags |= NPMAF_USE_CMT;
+   }
+   if(flow->getTrafficSpec().UseRP) {
+      addFlowMsg->Header.Flags |= NPMAF_USE_RP;
+   }
+
    addFlowMsg->Header.Length = htons(addFlowMsgSize);
    addFlowMsg->MeasurementID = hton64(flow->getMeasurementID());
    addFlowMsg->FlowID        = htonl(flow->getFlowID());
@@ -730,6 +737,7 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
          trafficSpec.OutboundFrameRate[i] = networkToDouble(addFlowMsg->FrameRate[i]);
          trafficSpec.OutboundFrameSize[i] = networkToDouble(addFlowMsg->FrameSize[i]);
       }
+      trafficSpec.ErrorOnAbort             = false;
       trafficSpec.OutboundFrameRateRng     = addFlowMsg->FrameRateRng;
       trafficSpec.OutboundFrameSizeRng     = addFlowMsg->FrameSizeRng;
       trafficSpec.MaxMsgSize               = ntohs(addFlowMsg->MaxMsgSize);
@@ -745,6 +753,8 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
       for(size_t i = 0;i < startStopEvents;i++) {
          trafficSpec.OnOffEvents.insert(ntohl(addFlowMsg->OnOffEvent[i]));
       }
+      trafficSpec.UseCMT = (addFlowMsg->Header.Flags & NPMAF_USE_CMT)    ? true : false;
+      trafficSpec.UseRP  = (addFlowMsg->Header.Flags & NPMAF_USE_RP)     ? true : false;
 
       Flow* flow = new Flow(ntoh64(addFlowMsg->MeasurementID), ntohl(addFlowMsg->FlowID),
                             ntohs(addFlowMsg->StreamID), trafficSpec,
@@ -808,8 +818,10 @@ static bool handleNetPerfMeterStart(MessageReader*                  messageReade
    }
    const uint64_t measurementID = ntoh64(startMsg->MeasurementID);
 
-   std::cout << std::endl << "Starting measurement "
-               << format("$%llx", (unsigned long long)measurementID) << " ..." << std::endl;
+   if(gOutputVerbosity >= NPFOV_STATUS) {
+      std::cout << std::endl << "Starting measurement "
+                << format("$%llx", (unsigned long long)measurementID) << " ..." << std::endl;
+   }
 
    OutputFileFormat scalarFileFormat = OFF_Plain;
    if(startMsg->Header.Flags & NPMSF_COMPRESS_SCALARS) {
@@ -854,9 +866,11 @@ static bool handleNetPerfMeterStop(MessageReader*                 messageReader,
    }
    const uint64_t measurementID = ntoh64(stopMsg->MeasurementID);
 
-   std::cout << std::endl << "Stopping measurement "
-             << format("$%llx", (unsigned long long)measurementID)
-             << " ..." << std::endl;
+   if(gOutputVerbosity >= NPFOV_STATUS) {
+      std::cout << std::endl << "Stopping measurement "
+               << format("$%llx", (unsigned long long)measurementID)
+               << " ..." << std::endl;
+   }
 
    // ====== Stop flows =====================================================
    FlowManager::getFlowManager()->lock();
