@@ -1,4 +1,4 @@
-/* $Id: flow.cc 645 2010-07-19 19:04:50Z dreibh $
+/* $Id: flow.cc 791 2010-11-17 09:34:38Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -20,6 +20,7 @@
  */
 
 #include "flow.h"
+#include "control.h"
 #include "transfer.h"
 
 #include <signal.h>
@@ -215,6 +216,7 @@ bool FlowManager::startMeasurement(const uint64_t           measurementID,
       }
    }
    unlock();
+   CPULoadStats.update();
 
    return(success);
 }
@@ -225,6 +227,7 @@ void FlowManager::stopMeasurement(const uint64_t           measurementID,
                                   const bool               printFlows,
                                   const unsigned long long now)
 {
+   CPULoadStats.update();
    lock();
    // We make a two-staged stopping process here:
    // In stage 0, the flows' sender threads are told to stop.
@@ -415,26 +418,68 @@ void FlowManager::handleEvents(const unsigned long long now)
          LastDisplayEvent  = now;
       }
 
-      // ====== Print bandwidth information line ============================
-      if(DisplayOn) {
-         const double       duration      = (now - LastDisplayEvent) / 1000000.0;
-         const unsigned int totalDuration = (unsigned int)((now - FirstDisplayEvent) / 1000000ULL);
+      // ====== Print bandwidth/CPU information line ========================
+      if((DisplayOn) && (gOutputVerbosity >= NPFOV_BANDWIDTH_INFO)) {
+         static const unsigned int cpuDisplayLimit = 4;
+         static const char* colorOff         = "\x1b[0m";
+         static const char* colorDuration    = "\x1b[34m";
+         static const char* colorFlows       = "\x1b[32m";
+         static const char* colorTransmitted = "\x1b[33m";
+         static const char* colorReceived    = "\x1b[36m";
+         static const char* colorCPU         = "\x1b[0m\x1b[31m";
+         static const char* colorCPUHighLoad = "\x1b[31;47;21m";
+         const double       duration         = (now - LastDisplayEvent) / 1000000.0;
+         const unsigned int totalDuration    = (unsigned int)((now - FirstDisplayEvent) / 1000000ULL);
+
+         char cpuUtilization[160];
+         cpuUtilization[0] = 0x00;
+         CPUDisplayStats.update();
+         if(CPUDisplayStats.getNumberOfCPUs() <= cpuDisplayLimit) {
+            for(unsigned int i = 1; i <= CPUDisplayStats.getNumberOfCPUs(); i++) {
+               const unsigned int load = (unsigned int)rint(10 * CPUDisplayStats.getCpuUtilization(i));
+               char str[32];
+               snprintf((char*)&str, sizeof(str), "%s%s%3u.%u%%%s",
+                        (i > 1) ? "/" : "",
+                        (load < 900) ? colorCPU : colorCPUHighLoad,
+                        load / 10, load % 10,
+                        colorCPU);
+               safestrcat((char*)&cpuUtilization, str, sizeof(cpuUtilization));
+            }
+         }
+         const unsigned int load = (unsigned int)rint(10 * CPUDisplayStats.getCpuUtilization(0));
+         char str[48];
+         snprintf((char*)&str, sizeof(str), "%s%s%3u.%u%%%s%s",
+                  (CPUDisplayStats.getNumberOfCPUs() > cpuDisplayLimit) ? "" : " [",
+                  (load < 900) ? colorCPU : colorCPUHighLoad,
+                  load / 10, load % 10,
+                  colorCPU,
+                  (CPUDisplayStats.getNumberOfCPUs() > cpuDisplayLimit) ? " total" : "]");
+         safestrcat((char*)&cpuUtilization, str, sizeof(cpuUtilization));
 
          // NOTE: ostream/cout has race condition problem according to helgrind.
          //       => simply using stdout instead.
          fprintf(stdout,
-                 "\r<-- Duration: %2u:%02u:%02u   "
-                 "Flows: %u   "
-                 "Transmitted: %1.2f MiB at %1.1f Kbit/s   "
-                 "Received: %1.2f MiB at %1.1f Kbit/s -->\x1b[0K",
+                 "\r<-- %sDuration: %2u:%02u:%02u   "
+                 "%sFlows: %u   "
+                 "%sTransmitted: %1.2f MiB at %1.1f Kbit/s   "
+                 "%sReceived: %1.2f MiB at %1.1f Kbit/s   "
+                 "%sCPU: %s "
+                 "%s-->\x1b[0K",
+                 colorDuration,
                  totalDuration / 3600,
                  (totalDuration / 60) % 60,
                  totalDuration % 60,
+                 colorFlows,
                  (unsigned int)FlowSet.size(),
+                 colorTransmitted,
                  GlobalStats.TransmittedBytes / (1024.0 * 1024.0),
                  (duration > 0.0) ? (8 * RelGlobalStats.TransmittedBytes / (1000.0 * duration)) : 0.0,
+                 colorReceived,
                  GlobalStats.ReceivedBytes / (1024.0 * 1024.0),
-                 (duration > 0.0) ? (8 * RelGlobalStats.ReceivedBytes / (1000.0 * duration)) : 0.0);
+                 (duration > 0.0) ? (8 * RelGlobalStats.ReceivedBytes / (1000.0 * duration)) : 0.0,
+                 colorCPU,
+                 cpuUtilization,
+                 colorOff);
          fflush(stdout);
       }
 
@@ -469,12 +514,14 @@ void FlowManager::writeScalarStatistics(const uint64_t           measurementID,
             "scalar \"%s.flow[%u]\" \"Transmitted Bytes\"       %llu\n"
             "scalar \"%s.flow[%u]\" \"Transmitted Packets\"     %llu\n"
             "scalar \"%s.flow[%u]\" \"Transmitted Frames\"      %llu\n"
+            "scalar \"%s.flow[%u]\" \"Transmitted Bit Rate\"    %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Transmitted Byte Rate\"   %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Transmitted Packet Rate\" %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Transmitted Frame Rate\"  %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Received Bytes\"          %llu\n"
             "scalar \"%s.flow[%u]\" \"Received Packets\"        %llu\n"
             "scalar \"%s.flow[%u]\" \"Received Frames\"         %llu\n"
+            "scalar \"%s.flow[%u]\" \"Received Bit Rate\"       %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Received Byte Rate\"      %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Received Packet Rate\"    %1.6f\n"
             "scalar \"%s.flow[%u]\" \"Received Frame Rate\"     %1.6f\n"
@@ -482,12 +529,14 @@ void FlowManager::writeScalarStatistics(const uint64_t           measurementID,
             objectName.c_str(), flow->FlowID, flow->CurrentBandwidthStats.TransmittedBytes,
             objectName.c_str(), flow->FlowID, flow->CurrentBandwidthStats.TransmittedPackets,
             objectName.c_str(), flow->FlowID, flow->CurrentBandwidthStats.TransmittedFrames,
+            objectName.c_str(), flow->FlowID, (transmissionDuration > 0.0) ? 8ULL * flow->CurrentBandwidthStats.TransmittedBytes / transmissionDuration : 0.0,
             objectName.c_str(), flow->FlowID, (transmissionDuration > 0.0) ? flow->CurrentBandwidthStats.TransmittedBytes   / transmissionDuration : 0.0,
             objectName.c_str(), flow->FlowID, (transmissionDuration > 0.0) ? flow->CurrentBandwidthStats.TransmittedPackets / transmissionDuration : 0.0,
             objectName.c_str(), flow->FlowID, (transmissionDuration > 0.0) ? flow->CurrentBandwidthStats.TransmittedFrames  / transmissionDuration : 0.0,
             objectName.c_str(), flow->FlowID, flow->CurrentBandwidthStats.ReceivedBytes,
             objectName.c_str(), flow->FlowID, flow->CurrentBandwidthStats.ReceivedPackets,
             objectName.c_str(), flow->FlowID, flow->CurrentBandwidthStats.ReceivedFrames,
+            objectName.c_str(), flow->FlowID, (receptionDuration > 0.0) ? 8ULL * flow->CurrentBandwidthStats.ReceivedBytes / receptionDuration : 0.0,
             objectName.c_str(), flow->FlowID, (receptionDuration > 0.0) ? flow->CurrentBandwidthStats.ReceivedBytes   / receptionDuration : 0.0,
             objectName.c_str(), flow->FlowID, (receptionDuration > 0.0) ? flow->CurrentBandwidthStats.ReceivedPackets / receptionDuration : 0.0,
             objectName.c_str(), flow->FlowID, (receptionDuration > 0.0) ? flow->CurrentBandwidthStats.ReceivedFrames  / receptionDuration : 0.0
@@ -541,6 +590,22 @@ void FlowManager::writeScalarStatistics(const uint64_t           measurementID,
       objectName.c_str(), (totalDuration > 0.0) ? totalBandwidthStats.LostFrames  / totalDuration : 0.0
       );
    unlock();
+
+   // ====== Write CPU statistics ===========================================
+   for(unsigned int i = 1; i <= CPULoadStats.getNumberOfCPUs(); i++) {
+      for(unsigned int j = 0; j < CPULoadStats.getCpuStates(); j++) {
+         scalarFile.printf("scalar \"%s.CPU[%u]\" \"%s\" %1.3f\n",
+                           objectName.c_str(), i,
+                           CPULoadStats.getCpuStateName(j),
+                           CPULoadStats.getCpuStatePercentage(i, j));
+      }
+   }
+   for(unsigned int j = 0; j < CPULoadStats.getCpuStates(); j++) {
+      scalarFile.printf("scalar \"%s.totalCPU\" \"%s\" %1.3f\n",
+                        objectName.c_str(),
+                        CPULoadStats.getCpuStateName(j),
+                        CPULoadStats.getCpuStatePercentage(0, j));
+   }
 }
 
 
@@ -984,13 +1049,10 @@ void Flow::deactivate(const bool asyncStop)
             // timeout.
          }
          else {
-#ifdef __linux__
-#warning Using shutdown() instead of sctp_sengmsg() with SCTP_EOF flag.
             const int shutdownOkay = ext_shutdown(SocketDescriptor, 2);
-#else
-            const int shutdownOkay = ext_shutdown(SocketDescriptor, 2);
-//            const int shutdownOkay = sctp_sendmsg(SocketDescriptor, NULL, 0, NULL, 0, 0, SCTP_EOF, 0, 0, 0);
-#endif
+/*
+            const int shutdownOkay = sctp_sendmsg(SocketDescriptor, NULL, 0, NULL, 0, 0, SCTP_EOF, 0, 0, 0);
+*/
             if(shutdownOkay < 0) {
                perror("WARNING: Cannut shut association down");
             }
@@ -1121,8 +1183,8 @@ void Flow::run()
                } while(scheduleNextTransmissionEvent() <= now);
 
                if(TrafficSpec.Protocol == IPPROTO_UDP) {
-// ???????????                  result = true;   // Result is always true for UDP.
-                                   // result==false may e.g. be overload, etc.
+                  // Keep sending, even if there is a temporary failure.
+                  result = true;
                }
             }
          }
