@@ -1,4 +1,4 @@
-/* $Id: flow.h 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: flow.h 738 2010-09-02 07:21:40Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -29,6 +29,7 @@
 #include "flowtrafficspec.h"
 #include "defragmenter.h"
 #include "measurement.h"
+#include "cpustatus.h"
 #include "tools.h"
 
 #include <poll.h>
@@ -153,13 +154,16 @@ class FlowManager : public Thread
    unsigned long long               FirstDisplayEvent;
    unsigned long long               LastDisplayEvent;
    unsigned long long               NextDisplayEvent;
+   CPUStatus                        CPULoadStats;
    FlowBandwidthStats               GlobalStats;      // For displaying only
    FlowBandwidthStats               RelGlobalStats;   // For displaying only
+   CPUStatus                        CPUDisplayStats;  // For displaying only
 };
 
 
 class Flow : public Thread
 {
+   public:
    friend class FlowManager;
    enum FlowStatus {
       WaitingForStartup = 1,
@@ -205,6 +209,9 @@ class Flow : public Thread
    inline FlowStatus getInputStatus() const {
       return(InputStatus);
    }
+   inline bool isAcceptedIncomingFlow() const {
+      return(AcceptedIncomingFlow);
+   }
 
    inline void endOfInput() {
       InputStatus = Off;
