@@ -1,4 +1,4 @@
-/* $Id: flowtrafficspec.cc 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: flowtrafficspec.cc 728 2010-08-31 08:02:50Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -122,6 +122,12 @@ void FlowTrafficSpec::print(std::ostream& os) const
       os << "*0 ";
    }
    os << "}" << std::endl;
+   os << "      - Error on Abort:      "
+      << ((ErrorOnAbort == true) ? "yes" : "no") << std::endl;
+   os << "      - Use CMT:             "
+      << ((UseCMT == true) ? "yes" : "no") << std::endl;
+   os << "      - Use RP:              "
+      << ((UseRP == true) ? "yes" : "no") << std::endl;
 }
 
 
@@ -136,6 +142,9 @@ void FlowTrafficSpec::reset()
    ReliableMode             = 1.0;
    RetransmissionTrials     = ~0;
    RetransmissionTrialsInMS = true;
+   ErrorOnAbort             = true;
+   UseCMT                   = false;
+   UseRP                    = false;
    for(size_t i = 0;i < NETPERFMETER_RNG_INPUT_PARAMETERS;i++) {
       OutboundFrameRate[i] = 0.0;
       OutboundFrameSize[i] = 0.0;
