@@ -1,4 +1,4 @@
-/* $Id: flowtrafficspec.h 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: flowtrafficspec.h 728 2010-08-31 08:02:50Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -70,6 +70,10 @@ class FlowTrafficSpec
    uint8_t                OutboundFrameSizeRng;
    uint8_t                InboundFrameRateRng;
    uint8_t                InboundFrameSizeRng;
+
+   bool                   UseCMT;
+   bool                   UseRP;
+   bool                   ErrorOnAbort;
 };
 
 #endif
