@@ -1,4 +1,4 @@
-/* $Id: netperfmeter.cc 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: netperfmeter.cc 793 2010-11-17 15:08:10Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -245,6 +245,48 @@ static const char* parseTrafficSpecOption(const char*      parameters,
          trafficSpec.SndBufferSize = 4096;
       }
    }
+   else if(strncmp(parameters, "cmt=", 4) == 0) {
+      if(strncmp((const char*)&parameters[4], "on", 2) == 0) {
+         trafficSpec.UseCMT = true;
+         n = 4 + 2;
+      }
+      else if(strncmp((const char*)&parameters[4], "off", 3) == 0) {
+         trafficSpec.UseCMT = false;
+         n = 4 + 3;
+      }
+      else {
+         cerr << "ERROR: Invalid \"cmt\" setting: " << (const char*)&parameters[4] << "!" << std::endl;
+         exit(1);
+      }
+   }
+   else if(strncmp(parameters, "rp=", 3) == 0) {
+      if(strncmp((const char*)&parameters[3], "on", 2) == 0) {
+         trafficSpec.UseRP = true;
+         n = 3 + 2;
+      }
+      else if(strncmp((const char*)&parameters[3], "off", 3) == 0) {
+         trafficSpec.UseRP = false;
+         n = 3 + 3;
+      }
+      else {
+         cerr << "ERROR: Invalid \"rp\" setting: " << (const char*)&parameters[4] << "!" << std::endl;
+         exit(1);
+      }
+   }
+   else if(strncmp(parameters, "error_on_abort=", 15) == 0) {
+      if(strncmp((const char*)&parameters[15], "on", 2) == 0) {
+         trafficSpec.ErrorOnAbort = true;
+         n = 15 + 2;
+      }
+      else if(strncmp((const char*)&parameters[15], "off", 3) == 0) {
+         trafficSpec.ErrorOnAbort = false;
+         n = 15 + 3;
+      }
+      else {
+         cerr << "ERROR: Invalid \"error_on_abort\" setting: " << (const char*)&parameters[4] << "!" << std::endl;
+         exit(1);
+      }
+   }
    else if(sscanf(parameters, "description=%255[^:]s%n", (char*)&description, &n) == 1) {
       trafficSpec.Description = std::string(description);
       n = 12 + strlen(description);
@@ -302,15 +344,43 @@ static Flow* createFlow(Flow*                  previousFlow,
    // ====== Get FlowTrafficSpec ============================================
    FlowTrafficSpec trafficSpec;
    trafficSpec.Protocol = protocol;
-   parameters = parseNextEntry(parameters, (double*)&trafficSpec.OutboundFrameRate, &trafficSpec.OutboundFrameRateRng);
-   if(parameters) {
-      parameters = parseNextEntry(parameters, (double*)&trafficSpec.OutboundFrameSize, &trafficSpec.OutboundFrameSizeRng);
+   if(strncmp(parameters, "default", 7) == 0) {
+      trafficSpec.OutboundFrameRateRng = RANDOM_CONSTANT;
+      trafficSpec.OutboundFrameRate[0] = 0.0;
+      switch(trafficSpec.Protocol) {
+         case IPPROTO_TCP:
+            trafficSpec.OutboundFrameSize[0] = 1500 - 40 - 20;
+          break;
+         case IPPROTO_UDP:
+            trafficSpec.OutboundFrameSize[0] = 1500 - 40 - 8;
+            trafficSpec.OutboundFrameRate[0] = 25;
+          break;
+#ifdef HAVE_DCCP
+         case IPPROTO_DCCP:
+            trafficSpec.OutboundFrameSize[0] = 1500 - 40 - 40;   // 1420B for IPv6 via 1500B MTU!
+          break;
+#endif
+         default:
+            trafficSpec.OutboundFrameSize[0] = 1500 - 40 - 12 - 16;
+          break;
+      }
+      parameters = (char*)&parameters[7];
+      if(parameters[0] != 0x00) {
+         parameters++;
+         while( (parameters = parseTrafficSpecOption(parameters, trafficSpec, flowID)) ) { }
+      }
+   }
+   else {
+      parameters = parseNextEntry(parameters, (double*)&trafficSpec.OutboundFrameRate, &trafficSpec.OutboundFrameRateRng);
       if(parameters) {
-         parameters = parseNextEntry(parameters, (double*)&trafficSpec.InboundFrameRate, &trafficSpec.InboundFrameRateRng);
+         parameters = parseNextEntry(parameters, (double*)&trafficSpec.OutboundFrameSize, &trafficSpec.OutboundFrameSizeRng);
          if(parameters) {
-            parameters = parseNextEntry(parameters, (double*)&trafficSpec.InboundFrameSize, &trafficSpec.InboundFrameSizeRng);
+            parameters = parseNextEntry(parameters, (double*)&trafficSpec.InboundFrameRate, &trafficSpec.InboundFrameRateRng);
             if(parameters) {
-               while( (parameters = parseTrafficSpecOption(parameters, trafficSpec, flowID)) ) {
+               parameters = parseNextEntry(parameters, (double*)&trafficSpec.InboundFrameSize, &trafficSpec.InboundFrameSizeRng);
+               if(parameters) {
+                  while( (parameters = parseTrafficSpecOption(parameters, trafficSpec, flowID)) ) {
+                  }
                }
             }
          }
@@ -428,6 +498,42 @@ static Flow* createFlow(Flow*                  previousFlow,
               << " socket - " << strerror(errno) << "!" << endl;
          exit(1);
       }
+
+#ifdef SCTP_CMT_ON_OFF
+      struct sctp_assoc_value cmtOnOff;
+      cmtOnOff.assoc_id    = 0;
+      cmtOnOff.assoc_value = (flow->getTrafficSpec().UseCMT == true) ? 1 : 0;
+      if(ext_setsockopt(socketDescriptor, IPPROTO_SCTP, SCTP_CMT_ON_OFF, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
+         if(flow->getTrafficSpec().UseCMT == true) {
+            cerr << "ERROR: Failed to configure CMT usage on SCTP socket (SCTP_CMT_ON_OFF option) - "
+                 << strerror(errno) << "!" << endl;
+            exit(1);
+         }
+      }
+#else
+      if(flow->getTrafficSpec().UseCMT == true) {
+         cerr << "ERROR: CMT usage configured, but not supported by this system!" << endl;
+         exit(1);
+      }
+#endif
+#ifdef SCTP_RP_ON_OFF
+      struct sctp_assoc_value rpOnOff;
+      rpOnOff.assoc_id    = 0;
+      rpOnOff.assoc_value = (flow->getTrafficSpec().UseRP == true) ? 1 : 0;
+      if(ext_setsockopt(socketDescriptor, IPPROTO_SCTP, SCTP_RP_ON_OFF, &rpOnOff, sizeof(rpOnOff)) < 0) {
+         if(flow->getTrafficSpec().UseRP == true) {
+            cerr << "ERROR: Failed to configure RP usage on SCTP socket (SCTP_RP_ON_OFF option) - "
+                 << strerror(errno) << "!" << endl;
+            exit(1);
+         }
+      }
+#else
+      if(flow->getTrafficSpec().UseRP == true) {
+         cerr << "ERROR: RP usage configured, but not supported by this system!" << endl;
+         exit(1);
+      }
+#endif
+
       if(gOutputVerbosity >= NPFOV_STATUS) {
          cout << "okay <sd=" << socketDescriptor << "> ";
       }
