@@ -1,4 +1,4 @@
-/* $Id: netperfmeterpackets.h 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: netperfmeterpackets.h 728 2010-08-31 08:02:50Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -100,6 +100,8 @@ struct NetPerfMeterAddFlowMessage
 } __attribute__((packed));
 
 #define NPMAF_RTX_TRIALS_IN_MILLISECONDS (1 << 31)
+#define NPMAF_USE_CMT                    (1 << 30)
+#define NPMAF_USE_RP                     (1 << 29)
 
 
 struct NetPerfMeterRemoveFlowMessage
