@@ -1,6 +1,6 @@
-/* $Id: outputfile.h 555 2010-05-28 18:16:49Z dreibh $
+/* $Id: outputfile.h 702 2010-08-10 07:44:05Z dreibh $
  *
- * Network Performance Meter
+ * Plain/BZip2 File Output
  * Copyright (C) 2009-2010 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
@@ -44,9 +44,14 @@ class OutputFile
    OutputFile();
    ~OutputFile();
 
-   bool initialize(const char* name, const OutputFileFormat format);
-   bool finish(const bool closeFile = true);
+   bool initialize(const char*            name,
+                   const OutputFileFormat format,
+                   const unsigned int     compressionLevel = 9);
+   bool finish(const bool          closeFile    = true,
+               unsigned long long* bytesIn      = NULL,
+               unsigned long long* bytesOut     = NULL);
    bool printf(const char* str, ...);
+   bool write(const char* buffer, const size_t bufferLength);
 
    inline bool exists() const {
       return(File || BZFile);
