@@ -1,4 +1,4 @@
-/* $Id: transfer.cc 647 2010-07-20 11:35:28Z dreibh $
+/* $Id: transfer.cc 720 2010-08-26 07:29:11Z dreibh $
  *
  * Network Performance Meter
  * Copyright (C) 2009-2010 by Thomas Dreibholz
@@ -136,6 +136,18 @@ ssize_t sendNetPerfMeterData(Flow*                    flow,
    else {
       sent = ext_send(flow->getSocketDescriptor(), (char*)&outputBuffer, bytesToSend, 0);
    }
+
+   // ====== Check, whether flow has been aborted unintentionally ===========
+   if((sent < 0) &&
+      (errno != EAGAIN) &&
+      (!flow->isAcceptedIncomingFlow()) &&
+      (flow->getTrafficSpec().ErrorOnAbort) &&
+      (flow->getOutputStatus() == Flow::On)) {
+      std::cerr << "ERROR: Flow #" << flow->getFlowID() << " has been aborted - "
+                  << strerror(errno) << "!" << std::endl;
+      exit(1);
+   }
+
    return(sent);
 }
 
